# sample_two.py

import sys
import os
import platform
import time
import wx
import wx.adv
import wx.lib.fancytext as fancytext

# class MyTitleBar
# class MyAboutDlg
# class MyPopupMenu
# class MyWindow
# class MyButtonBox
# class MyPanel
# class MyFrame
# class MyApp

#-------------------------------------------------------------------------------

class MyTitleBar(wx.Control):
    """
    Thanks to Cody Precord.
    """
    def __init__(self, parent, label, size):
        super(MyTitleBar, self).__init__(parent, style=wx.BORDER_SIMPLE)

        #------------

        # Attributes.
        self.parent = parent
        self.label = label
        self.size = size

        #------------

        # Simplified init method.
        self.SetBackground()
        self.SetProperties(label, size)
        self.BindEvents()

    #-----------------------------------------------------------------------

    def SetBackground(self):
        """
        ...
        """

        self.SetBackgroundStyle(wx.BG_STYLE_CUSTOM)


    def SetProperties(self, label, size):
        """
        ...
        """

        self.label = label
        self.size = size

        self.label_font = self.GetFont()
        self.label_font.SetFamily(wx.SWISS)
        self.label_font.SetPointSize(size)
        self.label_font.SetWeight(wx.BOLD)
        self.SetFont(self.label_font)


    def BindEvents(self):
        """
        Bind some events to an events handler.
        """

        self.Bind(wx.EVT_PAINT, self.OnPaint)
        self.Bind(wx.EVT_LEFT_DCLICK, self.OnDoubleClick)  # View mask.
        self.Bind(wx.EVT_LEFT_DOWN, self.OnLeftDown)
        self.Bind(wx.EVT_LEFT_UP, self.OnLeftUp)


    def OnLeftDown(self, event):
        """
        ...
        """

        self.GetTopLevelParent().OnLeftDown(event)


    def OnLeftUp(self, event):
        """
        ...
        """

        self.GetTopLevelParent().OnLeftUp(event)


    def OnDoubleClick(self, event):
        """
        ...
        """

        self.GetTopLevelParent().OnDoubleClick(event)


    def SetLabel(self, label):
        """
        ...
        """

        self.label = label
        self.Refresh()


    def DoGetBestSize(self):
        """
        ...
        """

        dc = wx.ClientDC(self)
        dc.SetFont(self.GetFont())

        textWidth, textHeight = dc.GetTextExtent(self.label)
        spacing = 10
        totalWidth = textWidth + (spacing)
        totalHeight = textHeight + (spacing)

        best = wx.Size(totalWidth, totalHeight)
        self.CacheBestSize(best)

        return best


    def GetLabel(self):
        """
        ...
        """

        return self.label


    def GetLabelColor(self):
        """
        ...
        """

        return self.foreground


    def GetLabelSize(self):
        """
        ...
        """

        return self.size


    def SetLabelColour(self, colour):
        """
        ...
        """

        self.labelColour = colour


    def OnPaint(self, event):
        """
        ...
        """

        dc = wx.BufferedPaintDC(self)
        gcdc = wx.GCDC(dc)

        gcdc.Clear()

        # Setup the GraphicsContext.
        gc = gcdc.GetGraphicsContext()

        # Get the working size we can draw in.
        width, height = self.GetSize()

        # Use the GCDC to draw the text.
        gcdc.GradientFillLinear((0, 0, width, height+70), "#3D3D3D",
                                wx.SystemSettings.GetColour(wx.SYS_COLOUR_MENU),
                                wx.NORTH)

        # Get the system font.
        gcdc.SetFont(self.GetFont())

        textWidth, textHeight = gcdc.GetTextExtent(self.label)
        tposx, tposy = ((width/2)-(textWidth/2), (height/3)-(textHeight/3))

        tposx += 0
        tposy += 0

        # Set position and text color.
        gcdc.SetTextForeground("white")
        gcdc.DrawText(self.label, int(tposx-25), int(tposy+1))

        gcdc.SetTextForeground(self.labelColour)
        gcdc.DrawText(self.label, int(tposx-25), int(tposy))

#-------------------------------------------------------------------------------

class MyAboutDlg(wx.Frame):
    """
    Thanks to Robin Dunn.
    """
    def __init__(self, parent):
        style = (wx.FRAME_SHAPED | wx.NO_BORDER |
                 wx.CLIP_CHILDREN | wx.STAY_ON_TOP |
                 wx.SYSTEM_MENU | wx.CLOSE_BOX |
                 wx.NO_FULL_REPAINT_ON_RESIZE)
        wx.Frame.__init__(self,
                          parent,
                          id=-1,
                          title="About...",
                          style=style)

        #------------

        # Attributes.
        self.SetTransparent(0)
        self.opacity_in = 0
        self.opacity_out = 255
        self.deltaN = -70
        self.hasShape = False
        self.delta = wx.Point(0,0)

        #------------

        # Return application name.
        self.app_name = wx.GetApp().GetAppName()
        # Return bitmaps folder.
        self.bitmaps_dir = wx.GetApp().GetBitmapsDir()
        # Return icons folder.
        self.icons_dir = wx.GetApp().GetIconsDir()

        #------------

        # Simplified init method.
        self.SetProperties()
        self.OnTimerIn(self)
        self.CreateCtrls()
        self.BindEvents()

        #------------

        self.CenterOnParent(wx.BOTH)
        self.GetParent().Enable(False)

        #------------

        self.Show(True)

        #------------

        self.eventLoop = wx.GUIEventLoop()
        self.eventLoop.Run()

    #---------------------------------------------------------------------------

    def SetProperties(self):
        """
        Set the dialog properties (title, icon, transparency...).
        """

        frameIcon = wx.Icon(os.path.join(self.icons_dir,
                                         "icon_wxWidgets.ico"),
                            type=wx.BITMAP_TYPE_ICO)
        self.SetIcon(frameIcon)


    def OnTimerIn(self, evt):
        """
        Thanks to Pascal Faut.
        """

        self.timer1 = wx.Timer(self, -1)
        self.timer1.Start(1)
        self.Bind(wx.EVT_TIMER, self.AlphaCycle1, self.timer1)

        print("Fade-in was launched.")


    def OnTimerOut(self, evt):
        """
        Thanks to Pascal Faut.
        """

        self.timer2 = wx.Timer(self, -1)
        self.timer2.Start(1)
        self.Bind(wx.EVT_TIMER, self.AlphaCycle2, self.timer2)

        print("Fade-out was launched.")


    def AlphaCycle1(self, *args):
        """
        Thanks to Pascal Faut.
        """

        self.opacity_in += self.deltaN
        if self.opacity_in <= 0:
            self.deltaN = -self.deltaN
            self.opacity_in = 0

        if self.opacity_in >= 255:
            self.deltaN = -self.deltaN
            self.opacity_in = 255

            self.timer1.Stop()

        self.SetTransparent(self.opacity_in)

        print("Fade in = {}/255".format(self.opacity_in))


    def AlphaCycle2(self, *args):
        """
        Thanks to Pascal Faut.
        """

        self.opacity_out += self.deltaN
        if self.opacity_out >= 255:
            self.deltaN = -self.deltaN
            self.opacity_out = 255

        if self.opacity_out <= 0:
            self.deltaN = -self.deltaN
            self.opacity_out = 0

            self.timer2.Stop()

            wx.CallAfter(self.Destroy)

        self.SetTransparent(self.opacity_out)

        print("Fade out = {}/255".format(self.opacity_out))

        
    def CreateCtrls(self):
        """
        Make widgets for my dialog.
        """

        # Load a background bitmap.
        self.bmp = wx.Bitmap(os.path.join(self.bitmaps_dir,
                                          "skin_about.png"),
                             type=wx.BITMAP_TYPE_PNG)
        mask = wx.Mask(self.bmp, wx.RED)
        self.bmp.SetMask(mask)

        #------------

        self.SetClientSize((self.bmp.GetWidth(), self.bmp.GetHeight()))

        #------------

        if wx.Platform == "__WXGTK__":
            # wxGTK requires that the window be created before you can
            # set its shape, so delay the call to SetWindowShape until
            # this event.
            self.Bind(wx.EVT_WINDOW_CREATE, self.SetWindowShape)
        else:
            # On wxMSW and wxMac the window has already
            # been created, so go for it.
            self.SetWindowShape()


    def BindEvents(self):
        """
        Bind all the events related to my dialog.
        """

        # Bind some events to an events handler.
        self.Bind(wx.EVT_LEFT_UP, self.OnLeftUp)
        self.Bind(wx.EVT_LEFT_DOWN, self.OnLeftDown)
        self.Bind(wx.EVT_RIGHT_UP, self.OnCloseWindow)  # Panel right clic.
        self.Bind(wx.EVT_LEFT_DCLICK, self.OnDoubleClick)  # View mask.
        self.Bind(wx.EVT_ERASE_BACKGROUND, self.OnEraseBackground)
        self.Bind(wx.EVT_WINDOW_CREATE, self.SetWindowShape)
        self.Bind(wx.EVT_MOTION, self.OnMouseMove)
        self.Bind(wx.EVT_KEY_UP, self.OnKeyUp)
        self.Bind(wx.EVT_PAINT, self.OnPaint)
        self.Bind(wx.EVT_CLOSE, self.OnCloseWindow)


    def SetWindowShape(self, event=None):
        """
        ...
        """

        # Use the bitmap's mask to determine the region.
        r = wx.Region(self.bmp)
        self.hasShape = self.SetShape(r)


    def OnEraseBackground(self, event):
        """
        ...
        """

        dc = event.GetDC()
        if not dc:
            dc = wx.ClientDC(self)
            rect = self.GetUpdateRegion().GetBox()
            dc.SetClippingRect(rect)


    def OnDoubleClick(self, event):
        """
        ...
        """

        if self.hasShape:
            self.SetShape(wx.Region())
            self.hasShape = False
        else:
            self.SetWindowShape()


    def OnLeftDown(self, event):
        """
        ...
        """

        self.CaptureMouse()
        x, y = self.ClientToScreen(event.GetPosition())
        originx, originy = self.GetPosition()
        dx = x - originx
        dy = y - originy
        self.delta = ((dx, dy))


    def OnLeftUp(self, evt):
        """
        ...
        """

        if self.HasCapture():
            self.ReleaseMouse()


    def OnMouseMove(self, event):
        """
        ...
        """

        if event.Dragging() and event.LeftIsDown():
            x, y = self.ClientToScreen(event.GetPosition())
            fp = (x - self.delta[0], y - self.delta[1])
            self.Move(fp)


    def OnPaint(self, event):
        """
        ...
        """

        dc = wx.AutoBufferedPaintDCFactory(self)
        dc.DrawBitmap(self.bmp, 0, 0, True)

        #------------

        # These are strings.
        py_version = sys.version.split()[0]

        str1 = (('<font style="normal" family="default" color="yellow" size="10" weight="bold">'
                 'Programming : </font>'
                 '<font style="normal" family="default" color="black" size="10" weight="normal">'
                 'Python {}</font>').format(py_version))   # Python 3.6.5

        str2 = (('<font style="normal" family="default" color="red" size="10" weight="bold">'
                 'GUI toolkit : </font>'
                 '<font style="normal" family="default" color="black" size="10" weight="normal">'
                 'wxPython {}</font>').format(wx.VERSION_STRING))  # wxPython 4.0.2

        str3 = (('<font style="normal" family="default" color="brown" size="10" weight="bold">'
                 'Library : </font>'
                 '<font style="normal" family="default" color="black" size="10" weight="normal">'
                 '{}</font>').format(wx.GetLibraryVersionInfo().VersionString))   # wxWidgets 3.0.5

        str4 = (('<font style="normal" family="default" color="blue" size="10" weight="bold">'
                 'Operating system : </font>'
                 '<font style="normal" family="default" color="black" size="10" weight="normal">'
                 '{}</font>').format(platform.system()))   # Windows

        str5 = (('<font style="normal" family="default" color="white" size="9" weight="normal">'
                 '{}</font>').format(self.app_name))   # Custom Gui 2

        str6 = (('<font style="normal" family="default" color="dark blue" size="8" weight="normal">'
                 'Right clic or Esc for Exit</font>'))

        #------------

        # Return image size.
        bw, bh = self.bmp.GetWidth(), self.bmp.GetHeight()

        # Draw text.
        # Need width to calculate x position of str1.
        tw, th = fancytext.GetExtent(str1, dc)
        # Centered text.
        fancytext.RenderToDC(str1, dc, (bw-tw)/2, 30)

        #------------

        # Need width to calculate x position of str2.
        tw, th = fancytext.GetExtent(str2, dc)
        # Centered text.
        fancytext.RenderToDC(str2, dc, (bw-tw)/2, 50)

        #------------

        # Need width to calculate x position of str3.
        tw, th = fancytext.GetExtent(str3, dc)
        # Centered text.
        fancytext.RenderToDC(str3, dc, (bw-tw)/2, 70)

        #------------

        # Need width to calculate x position of str4.
        tw, th = fancytext.GetExtent(str4, dc)
        # Centered text.
        fancytext.RenderToDC(str4, dc, (bw-tw)/2, 90)

        #------------

        # Need width to calculate x position of str5.
        tw, th = fancytext.GetExtent(str5, dc)
        # Centered text.
        fancytext.RenderToDC(str5, dc, (bw-tw)/2, 130)

        #------------

        # Need width to calculate x position of str6.
        tw, th = fancytext.GetExtent(str6, dc)
        # Centered text.
        fancytext.RenderToDC(str6, dc, (bw-tw)/2, 195)


    def OnKeyUp(self, event):
        """
        ...
        """

        if event.GetKeyCode() == wx.WXK_ESCAPE:
            self.OnCloseWindow(event)

        event.Skip()


    def OnCloseWindow(self, event):
        """
        ...
        """

        self.GetParent().Enable(True)
        self.eventLoop.Exit()
        #self.Destroy()
        self.OnTimerOut(self)

#-------------------------------------------------------------------------------

class MyPopupMenu(wx.Menu):
    """
    Thanks to Robin Dunn.
    """
    def __init__(self, parent):
        wx.Menu.__init__(self)

        #------------

        # Attributes.
        self.parent = parent

        #------------

        # Simplified init method.
        self.CreatePopupMenu()
        self.BindEvents()

    #---------------------------------------------------------------------------

    def CreatePopupMenu(self, event=None):
        """
        This method is called by the base class when it needs to popup
        the menu for the default EVT_RIGHT_DOWN event.  Just create
        the menu how you want it and return it from this function,
        the base class takes care of the rest.
        """

        # Returns bitmaps folder.
        bitmaps_dir = wx.GetApp().GetBitmapsDir()

        #------------

        bmp = wx.Bitmap(os.path.join(bitmaps_dir,
                                     "item_about.png"),
                        type=wx.BITMAP_TYPE_PNG)

        item = wx.MenuItem(self, id=wx.ID_ABOUT, text=" About")
        item.SetBitmap(bmp)
        self.Append(item)
        self.AppendSeparator()

        #------------

        bmp = wx.Bitmap(os.path.join(bitmaps_dir,
                                     "item_exit.png"),
                        type=wx.BITMAP_TYPE_PNG)

        if True or "__WXMSW__" in wx.PlatformInfo:
            font = wx.SystemSettings.GetFont(wx.SYS_DEFAULT_GUI_FONT)
            font.SetWeight(wx.BOLD)

        item = wx.MenuItem(self, id=wx.ID_EXIT, text=" Exit")
        item.SetBitmap(bmp)
        item.SetFont(font)
        self.Append(item)

        return self


    def BindEvents(self):
        """
        Bind some events to an events handler.
        """

        # Bind the menu events to an events handler.
        self.Bind(wx.EVT_MENU, self.OnAbout, id=wx.ID_ABOUT)
        self.Bind(wx.EVT_MENU, self.OnClose, id=wx.ID_EXIT)


    def OnAbout(self, event):
        """
        ...
        """

        self.mainFrame = wx.GetApp().GetTopWindow()
        self.mainFrame.OnAbout(self)

        print("About icon was clicked.")


    def OnClose(self, event):
        """
        ...
        """

        self.mainFrame = wx.GetApp().GetTopWindow()
        self.mainFrame.OnTimerOut(self)

        print("Close icon was clicked.")

#-------------------------------------------------------------------------------

class MyWindow(wx.Control):
    """
    Thanks to Cody Precord.
    """
    def __init__(self, parent, label,
                 foreground, background,
                 normal, pressed=None):
        super(MyWindow, self).__init__(parent, -1,
                                       style=wx.BORDER_NONE)

        #------------

        # Attributes.
        self.label = label
        self.foreground = foreground
        self.background = background

        if wx.Platform == "__WXGTK__":
            self.color = "#9e9d9d"

        else:
            self.color = "#b1b1b0"

        self.normal = normal
        self.pressed = pressed

        self._clicked = False

        #------------

        self.region = wx.Region(normal, wx.Colour(0, 0, 0, 0))

        #------------

        # Simplified init method.
        self.SetProperties(label, foreground, background)
        self.BindEvents()

    #---------------------------------------------------------------------------

    def SetProperties(self, label, foreground, background):
        """
        ...
        """

        self.label = label
        self.foreground = foreground
        self.background = background
        self.SetBackgroundStyle(wx.BG_STYLE_CUSTOM)


    def BindEvents(self):
        """
        Bind some events to an events handler.
        """

        self.Bind(wx.EVT_SET_FOCUS, self.OnSetFocus)
        self.Bind(wx.EVT_KILL_FOCUS, self.OnKillFocus)
        self.Bind(wx.EVT_PAINT, self.OnPaint)
        self.Bind(wx.EVT_LEFT_DOWN, self.OnLeftDown)
        self.Bind(wx.EVT_LEFT_DCLICK, self.OnLeftDclick)
        self.Bind(wx.EVT_LEFT_UP, self.OnLeftUp)
        self.Bind(wx.EVT_MOTION, self.OnMotion)
        self.Bind(wx.EVT_LEAVE_WINDOW, self.OnLeave)
        self.Bind(wx.EVT_KEY_UP, self.OnKeyUp)


    def SetLabel(self, label):
        """
        ...
        """

        self.label = label
        self.Refresh()


    def DoGetBestSize(self):
        """
        ...
        """

        return self.normal.GetSize()


    def GetLabel(self):
        """
        ...
        """

        return self.label


    def GetLabelColor(self):
        """
        ...
        """

        return self.foreground


    def Enable(self, *args, **kwargs):
        """
        ...
        """

        super(MyWindow, self).Enable(*args, **kwargs)
        self.Refresh()


    def Disable(self, *args, **kwargs):
        """
        ...
        """

        super(MyWindow, self).Disable(*args, **kwargs)
        self.Refresh()


    def PostEvent(self):
        """
        ...
        """

        event = wx.CommandEvent()
        event.SetEventObject(self)
        event.SetEventType(wx.EVT_BUTTON.typeId)
        wx.PostEvent(self, event)


    def OnSize(self, event):
        """
        ...
        """

        event.Skip()
        self.Refresh()


    def Getbackground(self):
        """
        ...
        """

        return self.background


    def OnPaint(self, event):
        """
        ...
        """

        dc = wx.BufferedPaintDC(self)
        dc.Clear()
        gcdc = wx.GCDC(dc)

        # Set the background color.
        if wx.Platform == "__WXGTK__":
            gcdc.SetBackground(wx.Brush("grey55"))
        else:
            gcdc.SetBackground(wx.Brush(self.background))

        gcdc.Clear()

        # Get the working rectangle we can draw in.
        rect = self.GetClientRect()

        # Font size and style.
        fontSize = self.GetFont().GetPointSize()

        if wx.Platform == "__WXGTK__":
            boldFont = wx.Font(fontSize-1, wx.DEFAULT,
                               wx.NORMAL, wx.NORMAL, False, "")
        else:
            boldFont = wx.Font(fontSize+2, wx.DEFAULT,
                               wx.NORMAL, wx.BOLD, False, "")

        if wx.Platform == "__WXMSW__":
            pen = wx.Pen(self.color, 1, wx.SOLID)

        else:
            pen = wx.Pen(self.color, 1, wx.SOLID)

        gcdc.SetPen(pen)

        x, y = self.GetSize()
        # x, y , width, height, radius
        gcdc.DrawRoundedRectangle (0, 0, 71, 25, 3)

        bitmap = self.normal

        w, h = bitmap.GetWidth(), bitmap.GetHeight()

        if self.clicked:
            bitmap = self.pressed or bitmap
        if not self.IsEnabled():
            bitmap = self.normal or bitmap

        # Draw a bitmap with an alpha channel.
        # image, x, y, transparency.
        dc.DrawBitmap(bitmap, 0, 0, True)

        if wx.Platform == "__WXGTK__":
            # Add the Caption.
            # White text - Shadow.
            rect = wx.Rect(rect.x, rect.y+3,
                           rect.width, 22)

        else:
            rect = wx.Rect(rect.x, rect.y+3,
                           rect.width, 20)

        dc.SetFont(boldFont)
        dc.SetTextForeground(wx.WHITE)
        dc.DrawLabel(self.label, rect, wx.ALIGN_CENTER)

        if wx.Platform == "__WXGTK__":
            # Add the Caption.
            # Black text.
            rect = wx.Rect(rect.x, rect.y,
                           rect.width, 21)

        else:
            rect = wx.Rect(rect.x, rect.y,
                           rect.width, 19)

        gcdc.SetFont(boldFont)
        # Get the text color.
        dc.SetTextForeground(self.foreground)
        dc.DrawLabel(self.label, rect, wx.ALIGN_CENTER)


    def SetClicked(self, clicked):
        """
        ...
        """

        if clicked != self._clicked:
            self._clicked = clicked
            self.Refresh()


    def GetClicked(self):
        """
        ...
        """

        return self._clicked


    clicked = property(GetClicked, SetClicked)
    def OnLeftDown(self, event):
        """
        ...
        """

        x, y = event.GetPosition()
        if self.region.Contains(x, y):
            self.clicked = True
            self.SetFocus()


    def OnLeftDclick(self, event):
        """
        ...
        """

        self.OnLeftDown(event)


    def OnLeftUp(self, event):
        """
        ...
        """

        if self.clicked:
            x, y = event.GetPosition()
            if self.region.Contains(x, y):
                self.PostEvent()
        self.clicked = False


    def OnMotion(self, event):
        """
        ...
        """

        if self.clicked:
            x, y = event.GetPosition()
            if not self.region.Contains(x, y):
                self.clicked = False


    def OnLeave(self, event):
        """
        ...
        """

        self.clicked = False


    def OnSetFocus(self, event):
        """
        ...
        """

        self.color = "white"
        self.Refresh()


    def OnKillFocus(self, event):
        """
        ...
        """

        if wx.Platform == "__WXGTK__":
            self.color = "#9e9d9d"

        else:
            self.color = "#b1b1b0"

        self.Refresh()


    def OnKeyUp(self, event):
        """
        ...
        """

        if event.GetKeyCode() == wx.WXK_SPACE:
            self.PostEvent()
            return

        elif event.GetKeyCode() == wx.WXK_ESCAPE:
            self.GetTopLevelParent().OnCloseWindow(True)

        event.Skip()

#-------------------------------------------------------------------------------

class MyButtonBox(wx.Panel):
    """
    Thanks to Cody Precord.
    """
    def __init__(self, parent, caption):
        super(MyButtonBox, self).__init__(parent,
                                           style=wx.NO_BORDER |
                                           wx.TAB_TRAVERSAL)

        #------------

        # Attributes.
        self._caption = caption

        #------------

        # Simplified init method.
        self.DoLayout()

    #---------------------------------------------------------------------------

    def DoLayout(self):
        """
        ...
        """

        self._csizer = wx.BoxSizer(wx.VERTICAL)
        msizer = wx.BoxSizer(wx.HORIZONTAL)
        msizer.Add(self._csizer, 0 )
        self.SetSizer(msizer)


    def DoGetBestSize(self):
        """
        ...
        """

        size = super(MyButtonBox, self).DoGetBestSize()

        # Compensate for wide caption labels.
        tw = self.GetTextExtent(self._caption)[0]
        size.SetWidth(max(size.width, size.height))
        return size


    def AddItem(self, item):
        """
        Add a window or sizer item to the MyButtonBox.
        """

        self._csizer.Add(item, 0 )

#-------------------------------------------------------------------------------

class MyPanel(wx.Panel):
    """
    Thanks to Cody Precord.
    """
    def __init__(self, parent):
        wx.Panel.__init__(self, parent)

        #------------

        # Return application name.
        self.app_name = wx.GetApp().GetAppName()
        # Return bitmaps folder.
        self.bitmaps_dir = wx.GetApp().GetBitmapsDir()

        #------------
        
        # Create a timer.
        self.timer = wx.Timer(self)
        self.timer.Start(1000)
        self.Bind(wx.EVT_TIMER, self.OnTimer, self.timer)
        
        #------------

        # Simplified init method.
        self.SetProperties()
        self.CreateCtrls()
        self.BindEvents()

    #---------------------------------------------------------------------------

    def SetProperties(self):
        """
        ...
        """

        self.SetBackgroundStyle(wx.BG_STYLE_CUSTOM)


    def CreateCtrls(self):
        """
        ...
        """

        # Load a background bitmap.
        self.bitmap = wx.Bitmap(os.path.join(self.bitmaps_dir,
                                             "skin_main.png"),
                                type=wx.BITMAP_TYPE_PNG)

        #------------

        # Add titleBar.
        self.titleBar = MyTitleBar(self,
                                   label=self.app_name,
                                   size=10)
        self.titleBar.SetPosition((1, 1))
        self.titleBar.SetSize((258, 24))
        self.titleBar.SetLabelColour(wx.BLACK)
        self.titleBar.SetToolTip("This is a customized title bar.")

        #------------

        # Load an icon bitmap for titlebar.
        self.icon = wx.Bitmap(os.path.join(self.bitmaps_dir,
                                           "icon_app_2.png"),
                              type=wx.BITMAP_TYPE_PNG)

        self.ico = wx.StaticBitmap(self.titleBar, -1, self.icon)
        self.ico.SetPosition((5, 2))
        self.ico.SetToolTip("This is a customized icon.")

        self.ico.Bind(wx.EVT_RIGHT_DOWN, self.OnRightDown)
        self.ico.Bind(wx.EVT_LEFT_DOWN, self.OnRightDown)

        #------------
        #------------

        # Load a bitmap.
        self.logo  = wx.Bitmap(os.path.join(self.bitmaps_dir,
                                            "logo.png"),
                               type=wx.BITMAP_TYPE_PNG)

        #------------

        # Aluminium Buttons box (main).
        self.box1 = MyButtonBox(self, "")   # Button About.
        self.box2 = MyButtonBox(self, "")   # Button Quit.

        # Gloss Buttons box (titleBar).
        self.box3 = MyButtonBox(self.titleBar, "")   # Button Exit.
        self.box4 = MyButtonBox(self.titleBar, "")   # Button Reduce.

        #------------

        # Aluminium Buttons bitmap.
        bmp1 = wx.Bitmap(os.path.join(self.bitmaps_dir,
                                      "btn_alu_normal.png"),
                         type=wx.BITMAP_TYPE_PNG)

        bmp2 = wx.Bitmap(os.path.join(self.bitmaps_dir,
                                      "btn_alu_pressed.png"),
                         type=wx.BITMAP_TYPE_PNG)

        bmpa = wx.Bitmap(os.path.join(self.bitmaps_dir,
                                      "btn_alu_exit_normal.png"),
                         type=wx.BITMAP_TYPE_PNG)

        bmpb = wx.Bitmap(os.path.join(self.bitmaps_dir,
                                      "btn_alu_exit_selected.png"),
                         type=wx.BITMAP_TYPE_PNG)


        bmpc = wx.Bitmap(os.path.join(self.bitmaps_dir,
                                      "btn_alu_roll_normal.png"),
                         type=wx.BITMAP_TYPE_PNG)

        bmpd = wx.Bitmap(os.path.join(self.bitmaps_dir,
                                      "btn_alu_roll_selected.png"),
                         type=wx.BITMAP_TYPE_PNG)

        # Buttons panel.
        self.btn1 = MyWindow(self.box1, "About", "black", "#9e9d9d", bmp1, bmp2)
        self.btn1.SetToolTip("This is a customized aluminium button.")
        self.btn1.Bind(wx.EVT_BUTTON, self.OnBtnAbout)
        self.btn1.SetFocus() # Necessary for Linux.

        self.btn2 = MyWindow(self.box2, "Exit", "black", "#9e9d9d", bmp1, bmp2)
        self.btn2.SetToolTip("This is a customized aluminium button.")
        self.btn2.Bind(wx.EVT_BUTTON, self.OnBtnClose)

        self.btn3 = MyWindow(self.box3, "", "black", "#9e9d9d", bmpa, bmpb)
        self.btn3.SetToolTip("This is a customized gloss button.")
        self.btn3.Bind(wx.EVT_BUTTON, self.OnBtnClose)

        self.btn4 = MyWindow(self.box4, "", "black", "#9e9d9d", bmpc, bmpd)
        self.btn4.SetToolTip("This is a customized gloss button.")
        self.btn4.Bind(wx.EVT_BUTTON, self.OnBtnRoll)

        self.box1.AddItem(self.btn1)
        self.box2.AddItem(self.btn2)
        self.box3.AddItem(self.btn3)
        self.box4.AddItem(self.btn4)

        self.box1.SetPosition((20, 138))
        self.box1.SetSize((71, 25))

        self.box2.SetPosition((20, 173))
        self.box2.SetSize((71, 25))

        self.box3.SetPosition((214, 2))
        self.box3.SetSize((40, 19))

        self.box4.SetPosition((189, 2))
        self.box4.SetSize((25, 19))


    def BindEvents(self):
        """
        Bind some events to an events handler.
        """

        self.Bind(wx.EVT_PAINT, self.OnPaint)
        self.Bind(wx.EVT_ERASE_BACKGROUND, self.OnEraseBackground)


    def OnLeftDown(self, evt):
        """
        ...
        """

        print("Left down.")


    def OnRightDown(self, event):
        """
        ...
        """

        self.PopupMenu(MyPopupMenu(self), event.GetPosition())

        print("Right down.")


    def OnEraseBackground(self, event):
        """
        ...
        """

        dc = event.GetDC()
        if not dc:
            dc = wx.ClientDC(self)
            rect = self.GetUpdateRegion().GetBox()
            dc.SetClippingRect(rect)


    def OnBtnRoll(self, event):
        """
        ...
        """

        self.GetParent().OnRoll(True)

        print("Roll/unRoll button was clicked.")


    def OnBtnAbout(self, event):
        """
        ...
        """

        self.GetParent().OnAbout(True)

        print("About button was clicked.")


    def OnBtnClose(self, event):
        """
        ...
        """

        self.GetParent().OnCloseWindow(self)

        print("Close button was clicked.")


    def Draw(self, dc):
        """
        ...
        """

        # Draw a bitmap with an alpha channel
        # on top of the last group.
        # image, x, y, transparence
        dc.DrawBitmap(self.bitmap, 0, 0, True)

        #------------

        fontSize = self.GetFont().GetPointSize()

        # wx.Font(pointSize, family, style, weight, underline, faceName)
        if wx.Platform == "__WXGTK__":
            self.titleBoldFont = wx.Font(fontSize-2,
                                         wx.DEFAULT, wx.NORMAL,
                                         wx.BOLD, False, "")
            self.normalBoldFont = wx.Font(fontSize+18,
                                          wx.DEFAULT, wx.NORMAL,
                                          wx.NORMAL, False, "")
            self.normalFont = wx.Font(fontSize-2,
                                      wx.DEFAULT, wx.NORMAL,
                                      wx.NORMAL, False, "")

        else:
            self.titleBoldFont = wx.Font(fontSize+1,
                                         wx.DEFAULT, wx.NORMAL,
                                         wx.BOLD, False, "")
            self.normalBoldFont = wx.Font(fontSize+23,
                                          wx.DEFAULT, wx.NORMAL,
                                          wx.NORMAL, False, "")
            self.normalFont = wx.Font(fontSize+1,
                                      wx.DEFAULT, wx.NORMAL,
                                      wx.NORMAL, False, "")

        dc.SetFont(self.normalFont)
        dc.SetFont(self.normalBoldFont)

        #------------

        # Draw a bitmap with an alpha channel.
        # image, x, y, transparence
        dc.DrawBitmap(self.logo, 150, 165, True)

        #------------

        # Return image size.
        bw, bh = self.bitmap.GetWidth(), self.bitmap.GetHeight()

        # Draw line.
        dc.SetPen(wx.Pen("#c5c4c4"))
        # x, y, x, y
        dc.DrawLine(2, 25, 258, 25)
        
        # Draw text.
        dc.SetTextForeground("#494848")
        dc.SetFont(self.normalFont)
        date = time.strftime("%A %d %B %Y")
        tw, th = dc.GetTextExtent(date)
        dc.DrawText(date, int((bw-tw)/2), int(100))

        dc.SetTextForeground("#e3dbdb")
        dc.SetFont(self.normalBoldFont)
        hour = time.strftime("%H:%M:%S")   # "%I:%M:%S %p"
        tw, th = dc.GetTextExtent(hour)
        dc.DrawText(hour, int((bw-tw)/2), int(40))


    def OnTimer(self, event):
        """
        ...
        """
        
        dc = wx.BufferedDC(wx.ClientDC(self))
        self.Draw(dc)
        self.Refresh()   # Mac


    def OnPaint(self, event):
        """
        ...
        """

        dc = wx.BufferedPaintDC(self)
        dc.Clear()

        self.Draw(dc)

#-------------------------------------------------------------------------------

class MyFrame(wx.Frame):
    """
    Thanks to Robin Dunn.
    """
    def __init__(self):
        style = (wx.CLIP_CHILDREN | wx.CLOSE_BOX |
                 wx.MINIMIZE_BOX | wx.SYSTEM_MENU |
                 wx.NO_BORDER | wx.FRAME_SHAPED |
                 wx.NO_FULL_REPAINT_ON_RESIZE)
        wx.Frame.__init__(self, None, -1, title="", style=style)

        #------------

        # Attributes.
        self.SetTransparent(0)
        self.opacity_in = 0
        self.opacity_out = 250
        self.deltaN = -3
        self.hasShape = False
        self.delta = wx.Point(0,0)

        #------------

        # Return application name.
        self.app_name = wx.GetApp().GetAppName()
        # Return bitmaps folder.
        self.bitmaps_dir = wx.GetApp().GetBitmapsDir()
        # Return icons folder.
        self.icons_dir = wx.GetApp().GetIconsDir()

        #------------

        # Simplified init method.
        self.SetProperties()
        self.OnTimerIn(self)
        self.CreateCtrls()
        self.BindEvents()
        self.DoLayout()

        #------------

        # Thanks to Ray Pasco.
        # Initialize to the current state.
        self.unrolledFrameClientSize_size = self.GetClientSize()
        self.isRolled = False

        #------------

        self.CenterOnScreen(wx.BOTH)

        #------------

        self.Show(True)

    #---------------------------------------------------------------------------

    def OnTimerIn(self, evt):
        """
        Thanks to Pascal Faut.
        """

        self.timer1 = wx.Timer(self, -1)
        self.timer1.Start(1)
        self.Bind(wx.EVT_TIMER, self.AlphaCycle1, self.timer1)

        print("Fade-in was launched.")


    def OnTimerOut(self, evt):
        """
        Thanks to Pascal Faut.
        """

        self.timer2 = wx.Timer(self, -1)
        self.timer2.Start(1)
        self.Bind(wx.EVT_TIMER, self.AlphaCycle2, self.timer2)

        print("Fade-out was launched.")


    def AlphaCycle1(self, *args):
        """
        Thanks to Pascal Faut.
        """

        self.opacity_in += self.deltaN
        if self.opacity_in <= 0:
            self.deltaN = -self.deltaN
            self.opacity_in = 0

        if self.opacity_in >= 250:
            self.deltaN = -self.deltaN
            self.opacity_in = 250

            self.timer1.Stop()

        self.SetTransparent(self.opacity_in)

        print("Fade in = {}/255".format(self.opacity_in))


    def AlphaCycle2(self, *args):
        """
        Thanks to Pascal Faut.
        """

        self.opacity_out += self.deltaN
        if self.opacity_out >= 250:
            self.deltaN = -self.deltaN
            self.opacity_out = 250

        if self.opacity_out <= 0:
            self.deltaN = -self.deltaN
            self.opacity_out = 0

            self.timer2.Stop()
            wx.CallAfter(self.Destroy)
            wx.Exit()
            
        self.SetTransparent(self.opacity_out)
        
        print("Fade out = {}/255".format(self.opacity_out))


    def SetProperties(self):
        """
        ...
        """

        self.SetTitle(self.app_name)
        self.SetBackgroundStyle(wx.BG_STYLE_CUSTOM)

        frameIcon = wx.Icon(os.path.join(self.icons_dir,
                                         "icon_wxWidgets.ico"),
                            type=wx.BITMAP_TYPE_ICO)
        self.SetIcon(frameIcon)


    def CreateCtrls(self):
        """
        ...
        """

        self.panel = MyPanel(self)

        #------------

        # Load a background bitmap.
        self.bmp = wx.Bitmap(os.path.join(self.bitmaps_dir,
                                          "skin_main.png"),
                             type=wx.BITMAP_TYPE_PNG)
        mask = wx.Mask(self.bmp, wx.RED)
        self.bmp.SetMask(mask)

        self.SetClientSize((self.bmp.GetWidth(), self.bmp.GetHeight()))

        #------------

        if wx.Platform == "__WXGTK__":
            # wxGTK requires that the window be created before you can
            # set its shape, so delay the call to SetWindowShape until
            # this event.
            self.Bind(wx.EVT_WINDOW_CREATE, self.SetWindowShape)
        else:
            # On wxMSW and wxMac the window has
            # already been created, so go for it.
            self.SetWindowShape()

        # Create an accelerator table.
        self.accel_tbl = wx.AcceleratorTable([(wx.ACCEL_CTRL, ord('X'), wx.ID_EXIT),
                                              (wx.ACCEL_CTRL, ord('A'), wx.ID_ABOUT)
                                             ])
        self.SetAcceleratorTable(self.accel_tbl)


    def BindEvents(self):
        """
        Bind some events to an events handler.
        """

        self.panel.Bind(wx.EVT_LEFT_DCLICK, self.OnDoubleClick)  # View mask.
        self.panel.Bind(wx.EVT_LEFT_DOWN, self.OnLeftDown)
        self.panel.Bind(wx.EVT_RIGHT_UP, self.OnCloseWindow)  # Panel right clic.

        self.Bind(wx.EVT_ERASE_BACKGROUND, self.OnEraseBackground)
        self.Bind(wx.EVT_CLOSE, self.OnCloseWindow)
        self.Bind(wx.EVT_LEFT_UP, self.OnLeftUp)
        self.Bind(wx.EVT_MOTION, self.OnMouseMove)

        self.Bind(wx.EVT_MENU, self.OnCloseWindow, id=wx.ID_EXIT)  # Ctrl+X.
        self.Bind(wx.EVT_MENU, self.OnAbout, id=wx.ID_ABOUT)


    def DoLayout(self):
        """
        ...
        """

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.panel, 1, wx.EXPAND)
        self.SetSizer(sizer)


    def OnRoll(self, event) :
        """
        Thanks to Ray Pasco.
        """

        if not bool(self.isRolled) :
        # Set the flag to the state we want regardless of whether
        # or not it's in currently in the opposite state.
           self.RollUnRoll(wantToRoll=True)

        elif self.isRolled :   # UnRoll.
           # Set the flag to the state we want regardless of whether
           # or not it's in currently in the opposite state.
           self.RollUnRoll(wantToRoll=False)


    def RollUnRoll(self, wantToRoll) :
        """
        Thanks to Ray Pasco.
        """

        # Thanks to Ray Pasco.
        # Save the current size only if the Frame is not rolled up.
        if not bool(self.isRolled) :
            self.unrolledFrameClientSize_size = self.GetClientSize()

        if bool(wantToRoll) :   # UnRoll.
            # Set size (25)
            self.SetClientSize((self.unrolledFrameClientSize_size[0], 25))
            # Set to match this new state.
            self.isRolled = True

        else :   # Roll
            self.SetClientSize(self.unrolledFrameClientSize_size)
            # Set to match this new state.
            self.isRolled = False


    def OnAbout(self, event):
        """
        ...
        """

        self.dialog = MyAboutDlg(self)


    def SetWindowShape(self, *evt):
        """
        ...
        """

        # Use the bitmap's mask to determine the region.
        r = wx.Region(self.bmp)
        self.hasShape = self.SetShape(r)


    def OnEraseBackground(self, event):
        """
        ...
        """

        dc = event.GetDC()
        if not dc:
            dc = wx.ClientDC(self)
            rect = self.GetUpdateRegion().GetBox()
            dc.SetClippingRect(rect)


    def OnDoubleClick(self, event):
        """
        ...
        """

        if self.hasShape:
            self.SetShape(wx.Region())
            self.hasShape = False
        else:
            self.SetWindowShape()


    def OnLeftDown(self, event):
        """
        ...
        """

        self.CaptureMouse()
        x, y = self.ClientToScreen(event.GetPosition())
        originx, originy = self.GetPosition()
        dx = x - originx
        dy = y - originy
        self.delta = ((dx, dy))


    def OnLeftUp(self, evt):
        """
        ...
        """

        if self.HasCapture():
            self.ReleaseMouse()


    def OnMouseMove(self, event):
        """
        ...
        """

        if event.Dragging() and event.LeftIsDown():
            x, y = self.ClientToScreen(event.GetPosition())
            fp = (x - self.delta[0], y - self.delta[1])
            self.Move(fp)


    def OnIconize(self, event):
        """
        ...
        """

        self.Iconize()


    def OnCloseWindow(self, event):
        """
        Quit this application.
        """

        # self.Destroy()
        self.OnTimerOut(self)

        print("Exit application.")

#-------------------------------------------------------------------------------

class MyApp(wx.App):
    """
    Thanks to Andrea Gavana.
    """
    def OnInit(self):

        #------------

        self.locale = wx.Locale(wx.LANGUAGE_FRENCH)

        #------------

        self.SetAppName("Custom Gui 2")

        #------------

        self.installDir = os.path.split(os.path.abspath(sys.argv[0]))[0]

        #------------

        frame = MyFrame()
        self.SetTopWindow(frame)
        frame.Show(True)

        return True

    #---------------------------------------------------------------------------

    def GetInstallDir(self):
        """
        Returns the installation directory for my application.
        """

        return self.installDir


    def GetIconsDir(self):
        """
        Returns the icons directory for my application.
        """

        icons_dir = os.path.join(self.installDir, "icons")
        return icons_dir


    def GetBitmapsDir(self):
        """
        Returns the bitmaps directory for my application.
        """

        bitmaps_dir = os.path.join(self.installDir, "bitmaps")
        return bitmaps_dir

#-------------------------------------------------------------------------------

def main():
    app = MyApp(False)
    app.MainLoop()

#-------------------------------------------------------------------------------

if __name__ == "__main__" :
    main()
    
